class StatcordException(Exception):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class RequestFailure(StatcordException):
    def __init__(self, status: int, response: str):
        super().__init__("{}: {}".format(status, response))



__title__ = 'statcord.py'
__author__ = 'Statcord.com'
__license__ = 'MIT'
__copyright__ = 'Copyright 2020, Statcord.com'
__version__ = '1.0.0'

name = "statcord"

from collections import namedtuple
from .client import Client
from .exceptions import *

VersionInfo = namedtuple('VersionInfo', 'major minor micro releaselevel serial')

version_info = VersionInfo(major=1, minor=0, micro=0, releaselevel='final', serial=0)



import asyncio
from discord import User
import aiohttp
from .exceptions import *


class Client:

    def __init__(self, bot, key):
        self.bot = bot
        self.key = key
        self.session = None
        self.base = "https://statcord.com/apollo/"

    def __session_init(self):
        if self.session is None:
            self.session = aiohttp.ClientSession()
    
    def __headers(self):
        return {'content-type': 'application/json'}
    
    async def __handle_response(self, resp: aiohttp.ClientResponse) -> dict:

        status = resp.status
        response = await resp.text()

        try:
            json = await resp.json()
        except:
            json = {}

        # Error
        if status != 200:
            print(response)

        return json

    @property
    def servers(self):
        try:
            count = len(self.bot.guilds)
        except AttributeError:
            count = len(self.bot.servers)

        return count

    @property
    def users(self):
        count = len(self.bot.users)
        return count

    async def post_data(self):
        bot_id = str(self.bot.user.id)
        key = self.key
        servers = str(self.servers)
        users = str(self.users)
        data = {"id":bot_id,"key":key,"servers":servers,"users":users}
        self.__session_init()
        async with self.session.post(url=self.base + "post/stats", json=data, headers=self.__headers()) as resp:
            return await self.__handle_response(resp)


    def start_loop(self):
        self.bot.loop.create_task(self.__loop())

    async def __loop(self):
        """
        The internal loop used for automatically posting server/guild count stats
        """
        await self.bot.wait_until_ready()
        while not self.bot.is_closed():
            await self.post_data()
            await asyncio.sleep(1800)
